/****    alert.c    ****    simple alert box implementation    ****/

/**********************************************************************
*  Copyright (c) 1990, 1991, 1992 Iris Computing Laboratories.
*
*  This software is provided for demonstration purposes only.  As
*  freely-distributed, modifiable source code, this software carries
*  absolutely no warranty.  Iris Computing Laboratories disclaims
*  all warranties for this software, including any implied warranties
*  of merchantability and fitness, and shall not be liable for
*  damages of any type resulting from its use.
*  Permission to use, copy, modify, and distribute this source code
*  for any purpose and without fee is hereby granted, provided that
*  the above copyright and this permission notice appear in all copies
*  and supporting documentation, and provided that Iris Computing
*  Laboratories not be named in advertising or publicity pertaining
*  to the redistribution of this software without specific, written
*  prior permission.
**********************************************************************/

#include "alert.h"



/*
Private globals:
*/

static Widget alertShellYesNoCancel, alertShellContinue;
static Widget alertYesNoCancel, alertContinue;


/*
alert_create_ync() creates a yes-no-cancel, three-button alert box.
*/

Widget alert_create_ync(parent, title)
Widget parent;
char *title;
{
	Arg args[3];
	int i;

	i = 0;
	XtSetArg(args[i], XtNallowShellResize, (XtArgVal) TRUE); i++;
	XtSetArg(args[i], XtNtitle, (XtArgVal) title); i++;
	alertShellYesNoCancel = XtCreatePopupShell("alertShellYesNoCancel",
		transientShellWidgetClass, parent, args, i);
	alertYesNoCancel = XtCreateManagedWidget("alertYesNoCancel",
		xiAlertWidgetClass, alertShellYesNoCancel, NULL, 0);
	XiAlertAddButton(alertYesNoCancel, "Yes", ALERT_YES, NULL, NULL);
	XiAlertAddButton(alertYesNoCancel, "No", ALERT_NO, NULL, NULL);
	XiAlertAddButton(alertYesNoCancel, "Cancel", ALERT_CANCEL, NULL, NULL);
	return alertShellYesNoCancel;
}	/* alert_create_ync */


/*
alert_create_continue() creates a one-button alert box.
*/

Widget alert_create_continue(parent, title)
Widget parent;
char *title;
{
	Arg args[3];
	int i;

	i = 0;
	XtSetArg(args[i], XtNallowShellResize, (XtArgVal) TRUE); i++;
	XtSetArg(args[i], XtNtitle, (XtArgVal) title); i++;
	alertShellContinue = XtCreatePopupShell("alertShellContinue",
		transientShellWidgetClass, parent, args, i);
	alertContinue = XtCreateManagedWidget("alertContinue",
		xiAlertWidgetClass, alertShellContinue, NULL, 0);
	XiAlertAddButton(alertContinue, "Continue", NULL, NULL, NULL);
	return alertShellContinue;
}	/* alert_create_continue */


/*
alert_delete() cancels an alert window.
*/

void alert_delete(w)
Widget w;
{
	if (w == alertShellYesNoCancel)
		XiAlertCancel(alertYesNoCancel, ALERT_CANCEL);
	else if (w == alertShellContinue)
		XiAlertCancel(alertContinue, ALERT_CANCEL);
}	/* alert_delete */


/*
alert() pops up a message window.
*/

void alert(msg)
char *msg;
{
	XiAlertPopup(alertContinue, msg, XiNOBEEP);
}	/* alert */


/*
alert_query() pops up a dialog window.
*/

int alert_query(msg)
char *msg;
{
	return XiAlertPopup(alertYesNoCancel, msg, XiNOBEEP);
}	/* alert_query */


/*
set_alert_reset_proper_keyboard_focus() establishes a pointer to
the client's procedure that handles keyboard focus operations.
See the procedure alert_reset_proper_keyboard_focus().
*/

void set_alert_reset_proper_keyboard_focus(rkfp)
void (*rkfp)();
{
	XiAlertResetProperKeyboardFocus(rkfp);
}	/* set_alert_reset_proper_keyboard_focus */

