/****    xcctool.c    ****/

/**********************************************************************
*  Copyright (c) 1990, 1991, 1992 Iris Computing Laboratories.
*
*  This software is provided for demonstration purposes only.  As
*  freely-distributed, modifiable source code, this software carries
*  absolutely no warranty.  Iris Computing Laboratories disclaims
*  all warranties for this software, including any implied warranties
*  of merchantability and fitness, and shall not be liable for
*  damages of any type resulting from its use.
*  Permission to use, copy, modify, and distribute this source code
*  for any purpose and without fee is hereby granted, provided that
*  the above copyright and this permission notice appear in all copies
*  and supporting documentation, and provided that Iris Computing
*  Laboratories not be named in advertising or publicity pertaining
*  to the redistribution of this software without specific, written
*  prior permission.
**********************************************************************/


#include "xcctool.h"
#include "alert.h"
#include "cctool.h"
#include "editor.h"


/*
Private functions:
*/

static void process_resources();
static void create_main_window_widgets();
static void create_compile_widgets();
static void create_error_widgets();
static void create_make_widgets();
static void initialize_shell_structures();
static int process_wm_event();
static void set_wm_protocols();
static void WMProtocolsAction();
static void Quit(), Compile(), Make(),
	CompileDone(), MakeDone(), ErrorDone(),
	CompileCompile(), CompileEdit(), CompileError(),
	PreviousError(), NextError(), MakeMake();


/*
Global widgets (for callbacks):
*/


Widget yncShell, contShell, compileShell, errorShell, makeShell, topLevel,
	formCompile, formError, formMake, errorMsgBox,
	buttonBoxMain, buttonBoxCompile, buttonBoxError, buttonBoxMake,
	vPaneCompile, vPaneError, vPaneMake,
	buttonQuit, buttonCompile, buttonMake,
	buttonDoneCompile, buttonEditCompile, buttonCompileCompile,
	buttonErrorCompile,
	buttonDoneError, buttonNextError, buttonPreviousError,
	buttonDoneMake, buttonMakeMake,
	compilerBoxCompile, optionsBoxCompile, librariesBoxCompile,
	directoryBoxCompile, filenameBoxCompile,
	cflagsBoxMake, librariesBoxMake, optionsBoxMake, ldflagsBoxMake,
	directoryBoxMake, filenameBoxMake;


/*
Private globals (for callbacks):
*/

static Pixmap xcctool_icon_pixmap;		/* freed by Quit() */
static Pixmap xcompile_icon_pixmap;		/* freed by Quit() */
static Pixmap xerror_icon_pixmap;		/* freed by Quit() */
static Pixmap xmake_icon_pixmap;		/* freed by Quit() */
static Pixmap xedit_icon_pixmap;		/* freed by Quit() */


/*
Private action globals:
*/

static Atom WM_DELETE_WINDOW;

static char app_trans[] =
	"<ClientMessage> WM_PROTOCOLS : WMProtocolsAction()";

static XtActionsRec app_actions[] = {
	{"WMProtocolsAction", WMProtocolsAction},
};


/*
Private resource globals:
*/

static XtResource resources[] = {
	{XtNeditRows, XtCEditRows, XtRInt, sizeof(int),
		XtOffset(ApplicationDataPtr, edit_rows), XtRImmediate,
		(caddr_t) DEFAULT_EDIT_ROWS},
	{XtNeditColumns, XtCEditColumns, XtRInt, sizeof(int),
		XtOffset(ApplicationDataPtr, edit_columns), XtRImmediate,
		(caddr_t) DEFAULT_EDIT_COLUMNS},
	{XtNeditSize, XtCEditSize, XtRInt, sizeof(int),
		XtOffset(ApplicationDataPtr, edit_size), XtRImmediate,
		(caddr_t) DEFAULT_EDIT_SIZE},
	{XtNeditFont, XtCEditFont, XtRFontStruct, sizeof(XFontStruct *),
		XtOffset(ApplicationDataPtr, edit_font),
		XtRString, "XtDefaultFont"},
	{XtNlibraries, XtCLibraries, XtRString, sizeof(char *),
		XtOffset(ApplicationDataPtr, cc_libraries),
		XtRString, DEFAULT_CC_LIBRARIES},
};



/*
main() creates the main widgets, sets up the alert boxes, initializes
data structures, etc.
*/

main(argc, argv)
int argc;
char *argv[];
{
	Arg args[5];
	int i, edit_width, edit_height;
	ApplicationData edit_data;

	i = 0;
	XtSetArg(args[i], XtNallowShellResize, (XtArgVal) FALSE); i++;
	topLevel = XtInitialize(argv[0], "XCctool", args, i,
		&argc, argv);
	process_resources(&edit_data);
	i = 0;
	XtSetArg(args[i], XtNinput, (XtArgVal) TRUE); i++;
	XtSetValues(topLevel, args, i);

	create_main_window_widgets();
	edit_width = edit_data.edit_columns *
		edit_data.edit_font->max_bounds.width;
	edit_height = edit_data.edit_rows *
		(edit_data.edit_font->max_bounds.ascent +
		edit_data.edit_font->max_bounds.descent);
	create_edit_window(edit_width, edit_height, edit_data.edit_size,
		edit_data.edit_font);
	create_compile_widgets(edit_data);
	create_error_widgets();
	create_make_widgets();
	yncShell = alert_create_ync(topLevel, "xcctool");
	contShell = alert_create_continue(topLevel, "xcctool");


	XtAddActions(app_actions, XtNumber(app_actions));
	XtAugmentTranslations(topLevel, XtParseTranslationTable(app_trans));
	XtAugmentTranslations(compileShell, XtParseTranslationTable(app_trans));
	XtAugmentTranslations(editShell, XtParseTranslationTable(app_trans));
	XtAugmentTranslations(errorShell, XtParseTranslationTable(app_trans));
	XtAugmentTranslations(makeShell, XtParseTranslationTable(app_trans));
	XtAugmentTranslations(yncShell, XtParseTranslationTable(app_trans));
	XtAugmentTranslations(contShell, XtParseTranslationTable(app_trans));

	set_edit_save_proc(save_proc);
	initialize_shell_structures();
	initialize_cctool_structures();

	XtRealizeWidget(topLevel);
	XtRealizeWidget(compileShell);
	XtRealizeWidget(errorShell);
	XtRealizeWidget(editShell);
	XtRealizeWidget(makeShell);
	XtRealizeWidget(yncShell);
	XtRealizeWidget(contShell);

	set_wm_protocols();

	XtMainLoop();
}	/* main */


/*
process_resources() reads dimensions for the edit window,
checks their bounds, and exits with warning message(s),
if there are problems.
*/

static void process_resources(edit_data)
ApplicationData *edit_data;
{
	XtGetApplicationResources(topLevel, edit_data, resources,
		XtNumber(resources), NULL, 0);
	if (edit_data->edit_rows < 1 ||
			edit_data->edit_rows > MAX_EDIT_ROWS) {
		fprintf(stderr, "xcctool: Illegal row dimension: %d\n",
			edit_data->edit_rows);
		fprintf(stderr,
			"Number of rows must be in the interval: [1,%d]\n",
			MAX_EDIT_ROWS);
		exit(-1);
	}
	if (edit_data->edit_columns < 1 ||
			edit_data->edit_columns > MAX_EDIT_COLUMNS) {
		fprintf(stderr, "xcctool: Illegal column dimension: %d\n",
			edit_data->edit_columns);
		fprintf(stderr,
			"Number of columns must be in the interval: [1,%d]\n",
			MAX_EDIT_COLUMNS);
		exit(-1);
	}
	if (edit_data->edit_size < 2 ||
			edit_data->edit_size > MAX_EDIT_SIZE) {
		fprintf(stderr, "xcctool: Illegal edit buffer size: %d\n",
			edit_data->edit_size);
		fprintf(stderr,
			"Number of rows must be in the interval: [2,%d]\n",
			MAX_EDIT_SIZE);
		exit(-1);
	}
}	/* process_resources */


/*
create_main_window_widgets() creates/initializes the widgets that
constitute the top-level, main window.
*/

static void create_main_window_widgets()
{
	Arg args[10];
	int i;

	i = 0;
	buttonBoxMain = XtCreateManagedWidget("buttonBoxMain",
		formWidgetClass, topLevel, NULL, 0);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Quit"); i++;
	buttonQuit = XtCreateManagedWidget("buttonQuit",
		commandWidgetClass, buttonBoxMain, args, i);
	XtAddCallback(buttonQuit, XtNcallback, Quit, NULL);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Compile"); i++;
	XtSetArg(args[i], XtNfromHoriz, (XtArgVal) buttonQuit); i++;
	buttonCompile = XtCreateManagedWidget("buttonCompile",
		commandWidgetClass, buttonBoxMain, args, i);
	XtAddCallback(buttonCompile, XtNcallback, Compile, NULL);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Make"); i++;
	XtSetArg(args[i], XtNfromHoriz, (XtArgVal) buttonCompile); i++;
	buttonMake = XtCreateManagedWidget("buttonMake",
		commandWidgetClass, buttonBoxMain, args, i);
	XtAddCallback(buttonMake, XtNcallback, Make, NULL);
}	/* create_main_window_widgets */


/*
create_compile_widgets() creates/initializes the widgets
in the pop-up compiler frame.
*/

static void create_compile_widgets(edit_data)
ApplicationData edit_data;
{
	Arg args[10];
	int i;

	i = 0;
	XtSetArg(args[i], XtNallowShellResize, (XtArgVal) FALSE); i++;
	XtSetArg(args[i], XtNmappedWhenManaged, (XtArgVal) FALSE); i++;
	compileShell = XtCreateApplicationShell("compileShell",
		topLevelShellWidgetClass, args, i);
	vPaneCompile = XtCreateManagedWidget("vPaneCompile",
#ifdef X11R3
		vPanedWidgetClass, compileShell, NULL, 0);
#else
		panedWidgetClass, compileShell, NULL, 0);
#endif
	buttonBoxCompile = XtCreateManagedWidget("buttonBoxCompile",
		boxWidgetClass, vPaneCompile, NULL, 0);
	i = 0;
	formCompile = XtCreateManagedWidget("formCompile",
		formWidgetClass, vPaneCompile, args, i);
	i = 0;
	XtSetArg(args[i], XtNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNtextWidth, (XtArgVal) 300); i++;
	XtSetArg(args[i], XiNlabel, (XtArgVal) "  Compiler:"); i++;
	XtSetArg(args[i], XiNtext, (XtArgVal) "cc"); i++;
	compilerBoxCompile = XtCreateManagedWidget("compilerBoxCompile",
		xiSimpleTextWidgetClass, formCompile, args, i);
	i = 0;
	XtSetArg(args[i], XtNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNtextWidth, (XtArgVal) 300); i++;
	XtSetArg(args[i], XiNlabel, (XtArgVal) "cc options:"); i++;
	XtSetArg(args[i], XiNtext, (XtArgVal) "-c"); i++;
	XtSetArg(args[i], XtNfromVert, (XtArgVal) compilerBoxCompile); i++;
	optionsBoxCompile = XtCreateManagedWidget("optionsBoxCompile",
		xiSimpleTextWidgetClass, formCompile, args, i);
	i = 0;
	XtSetArg(args[i], XtNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNtextWidth, (XtArgVal) 300); i++;
	XtSetArg(args[i], XiNlabel, (XtArgVal) " Libraries:"); i++;
	XtSetArg(args[i], XiNtext,
		(XtArgVal) edit_data.cc_libraries); i++;
	XtSetArg(args[i], XtNfromVert, (XtArgVal) optionsBoxCompile); i++;
	librariesBoxCompile = XtCreateManagedWidget("librariesBoxCompile",
		xiSimpleTextWidgetClass, formCompile, args, i);
	i = 0;
	XtSetArg(args[i], XtNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNtextWidth, (XtArgVal) 300); i++;
	XtSetArg(args[i], XiNlabel, (XtArgVal) " Directory:"); i++;
	XtSetArg(args[i], XiNtext, (XtArgVal) ""); i++;
	XtSetArg(args[i], XtNfromVert, (XtArgVal) librariesBoxCompile); i++;
	directoryBoxCompile = XtCreateManagedWidget("directoryBoxCompile",
		xiSimpleTextWidgetClass, formCompile, args, i);
	i = 0;
	XtSetArg(args[i], XtNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNtextWidth, (XtArgVal) 300); i++;
	XtSetArg(args[i], XiNlabel, (XtArgVal) "  Filename:"); i++;
	XtSetArg(args[i], XiNtext, (XtArgVal) ""); i++;
	XtSetArg(args[i], XtNfromVert, (XtArgVal) directoryBoxCompile); i++;
	filenameBoxCompile = XtCreateManagedWidget("filenameBoxCompile",
		xiSimpleTextWidgetClass, formCompile, args, i);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Done"); i++;
	buttonDoneCompile = XtCreateManagedWidget("buttonDoneCompile",
		commandWidgetClass, buttonBoxCompile, args, i);
	XtAddCallback(buttonDoneCompile, XtNcallback, CompileDone, NULL);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Edit"); i++;
	buttonEditCompile = XtCreateManagedWidget("buttonEditCompile",
		commandWidgetClass, buttonBoxCompile, args, i);
	XtAddCallback(buttonEditCompile, XtNcallback, CompileEdit, NULL);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Compile"); i++;
	buttonCompileCompile = XtCreateManagedWidget("buttonCompileCompile",
		commandWidgetClass, buttonBoxCompile, args, i);
	XtAddCallback(buttonCompileCompile, XtNcallback, CompileCompile, NULL);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Errors"); i++;
	buttonErrorCompile = XtCreateManagedWidget("buttonErrorCompile",
		commandWidgetClass, buttonBoxCompile, args, i);
	XtAddCallback(buttonErrorCompile, XtNcallback, CompileError, NULL);
}	/* create_compile_widgets */


/*
create_error_widgets() creates/initializes the widgets
in the pop-up error frame.
*/

static void create_error_widgets()
{
	Arg args[10];
	int i;

	i = 0;
	XtSetArg(args[i], XtNallowShellResize, (XtArgVal) FALSE); i++;
	XtSetArg(args[i], XtNmappedWhenManaged, (XtArgVal) FALSE); i++;
	errorShell = XtCreateApplicationShell("errorShell",
		topLevelShellWidgetClass, args, i);
	vPaneError = XtCreateManagedWidget("vPaneError",
#ifdef X11R3
		vPanedWidgetClass, errorShell, NULL, 0);
#else
		panedWidgetClass, errorShell, NULL, 0);
#endif
	buttonBoxError = XtCreateManagedWidget("buttonBoxError",
		boxWidgetClass, vPaneError, NULL, 0);
	i = 0;
	formError = XtCreateManagedWidget("formError",
		formWidgetClass, vPaneError, args, i);
	i = 0;
	XtSetArg(args[i], XtNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNtextBorderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNtextWidth, (XtArgVal) 600); i++;
	XtSetArg(args[i], XiNlabel, (XtArgVal) "Message:"); i++;
	XtSetArg(args[i], XiNtext,
		(XtArgVal) "*** no compilaton errors ***"); i++;
	errorMsgBox = XtCreateManagedWidget("errorMsgBox",
		xiSimpleTextWidgetClass, formError, args, i);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Done"); i++;
	buttonDoneError = XtCreateManagedWidget("buttonDoneError",
		commandWidgetClass, buttonBoxError, args, i);
	XtAddCallback(buttonDoneError, XtNcallback, ErrorDone, NULL);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Previous Error"); i++;
	buttonPreviousError = XtCreateManagedWidget("buttonPreviousError",
		commandWidgetClass, buttonBoxError, args, i);
	XtAddCallback(buttonPreviousError, XtNcallback, PreviousError, NULL);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Next Error"); i++;
	buttonNextError = XtCreateManagedWidget("buttonNextError",
		commandWidgetClass, buttonBoxError, args, i);
	XtAddCallback(buttonNextError, XtNcallback, NextError, NULL);
}	/* create_error_widgets */


/*
create_make_widgets() creates/initializes the widgets
in the pop-up make frame.
*/

static void create_make_widgets()
{
	static XiChoiceItem cflags_list[] = {
		{"Yes", YES, NULL, NULL},
		{"No", NO, NULL, NULL},
		{NULL, NULL, NULL, NULL},
	};
	Arg args[10];
	int i;

	i = 0;
	XtSetArg(args[i], XtNallowShellResize, (XtArgVal) FALSE); i++;
	XtSetArg(args[i], XtNmappedWhenManaged, (XtArgVal) FALSE); i++;
	makeShell = XtCreateApplicationShell("makeShell",
		topLevelShellWidgetClass, args, i);
	vPaneMake = XtCreateManagedWidget("vPaneMake",
#ifdef X11R3
		vPanedWidgetClass, makeShell, NULL, 0);
#else
		panedWidgetClass, makeShell, NULL, 0);
#endif
	buttonBoxMake = XtCreateManagedWidget("buttonBoxMake",
		boxWidgetClass, vPaneMake, NULL, 0);
	i = 0;
	formMake = XtCreateManagedWidget("formMake",
		formWidgetClass, vPaneMake, args, i);
	i = 0;
	XtSetArg(args[i], XiNlabel,
		(XtArgVal) "Pass \"cc options\" to $(CFLAGS)?"); i++;
	XtSetArg(args[i], XiNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNitemBorderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNexclusive, (XtArgVal) TRUE); i++;
	XtSetArg(args[i], XiNdefaultItem, (XtArgVal) 1); i++;
	XtSetArg(args[i], XiNlabelOnLeft, (XtArgVal) TRUE); i++;
	cflagsBoxMake = XtCreateManagedWidget("cflagsBoxMake",
		xiChoiceWidgetClass, formMake, args, i);
	XiChoiceAddItems(cflagsBoxMake, (caddr_t) cflags_list);
	i = 0;
	XtSetArg(args[i], XiNlabel,
		(XtArgVal) "Pass \"Libraries\" to $(LDFLAGS)?"); i++;
	XtSetArg(args[i], XiNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNitemBorderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNexclusive, (XtArgVal) TRUE); i++;
	XtSetArg(args[i], XiNdefaultItem, (XtArgVal) 1); i++;
	XtSetArg(args[i], XiNlabelOnLeft, (XtArgVal) TRUE); i++;
	XtSetArg(args[i], XtNfromVert, (XtArgVal) cflagsBoxMake); i++;
	librariesBoxMake = XtCreateManagedWidget("librariesBoxMake",
		xiChoiceWidgetClass, formMake, args, i);
	XiChoiceAddItems(librariesBoxMake, (caddr_t) cflags_list);
	i = 0;
	XtSetArg(args[i], XtNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNtextWidth, (XtArgVal) 300); i++;
	XtSetArg(args[i], XiNlabel, (XtArgVal) "      make options:"); i++;
	XtSetArg(args[i], XiNtext, (XtArgVal) ""); i++;
	XtSetArg(args[i], XtNfromVert, (XtArgVal) librariesBoxMake); i++;
	optionsBoxMake = XtCreateManagedWidget("optionsBoxMake",
		xiSimpleTextWidgetClass, formMake, args, i);
	i = 0;
	XtSetArg(args[i], XtNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNtextWidth, (XtArgVal) 300); i++;
	XtSetArg(args[i], XiNlabel, (XtArgVal) "$(LDFLAGS) options:"); i++;
	XtSetArg(args[i], XiNtext, (XtArgVal) ""); i++;
	XtSetArg(args[i], XtNfromVert, (XtArgVal) optionsBoxMake); i++;
	ldflagsBoxMake = XtCreateManagedWidget("ldflagsBoxMake",
		xiSimpleTextWidgetClass, formMake, args, i);
	i = 0;
	XtSetArg(args[i], XtNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNtextWidth, (XtArgVal) 300); i++;
	XtSetArg(args[i], XiNlabel, (XtArgVal) "         Directory:"); i++;
	XtSetArg(args[i], XiNtext, (XtArgVal) ""); i++;
	XtSetArg(args[i], XtNfromVert, (XtArgVal) ldflagsBoxMake); i++;
	directoryBoxMake = XtCreateManagedWidget("directoryBoxMake",
		xiSimpleTextWidgetClass, formMake, args, i);
	i = 0;
	XtSetArg(args[i], XtNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XiNtextWidth, (XtArgVal) 300); i++;
	XtSetArg(args[i], XiNlabel, (XtArgVal) "          Filename:"); i++;
	XtSetArg(args[i], XiNtext, (XtArgVal) ""); i++;
	XtSetArg(args[i], XtNfromVert, (XtArgVal) directoryBoxMake); i++;
	filenameBoxMake = XtCreateManagedWidget("filenameBoxMake",
		xiSimpleTextWidgetClass, formMake, args, i);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Done"); i++;
	buttonDoneMake = XtCreateManagedWidget("buttonDoneMake",
		commandWidgetClass, buttonBoxMake, args, i);
	XtAddCallback(buttonDoneMake, XtNcallback, MakeDone, NULL);
	i = 0;
	XtSetArg(args[i], XtNlabel, (XtArgVal) "Make"); i++;
	buttonMakeMake = XtCreateManagedWidget("buttonMakeMake",
		commandWidgetClass, buttonBoxMake, args, i);
	XtAddCallback(buttonMakeMake, XtNcallback, MakeMake, NULL);
}	/* create_make_widgets */


/*
initialize_shell_structures sets up miscellaneous structures
for the top-level widgets (currently, just the icon).
*/

static void initialize_shell_structures()
{

#include "xcctool.icon"
#include "xcompile.icon"
#include "xerror.icon"
#include "xmake.icon"
#include "xedit.icon"

	Arg args[3];

	xcctool_icon_pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
		RootWindowOfScreen(XtScreen(topLevel)), xcctool_bits,
		xcctool_width, xcctool_height);
	xcompile_icon_pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
		RootWindowOfScreen(XtScreen(topLevel)), xcompile_bits,
		xcompile_width, xcompile_height);
	xerror_icon_pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
		RootWindowOfScreen(XtScreen(topLevel)), xerror_bits,
		xerror_width, xerror_height);
	xmake_icon_pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
		RootWindowOfScreen(XtScreen(topLevel)), xmake_bits,
		xmake_width, xmake_height);
	xedit_icon_pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
		RootWindowOfScreen(XtScreen(topLevel)), xedit_bits,
		xedit_width, xedit_height);
	XtSetArg(args[0], XtNiconPixmap, (XtArgVal) xcctool_icon_pixmap);
	XtSetValues(topLevel, args, 1);
	XtSetArg(args[0], XtNiconPixmap, (XtArgVal) xcompile_icon_pixmap);
	XtSetValues(compileShell, args, 1);
	XtSetArg(args[0], XtNiconPixmap, (XtArgVal) xerror_icon_pixmap);
	XtSetValues(errorShell, args, 1);
	XtSetArg(args[0], XtNiconPixmap, (XtArgVal) xmake_icon_pixmap);
	XtSetValues(makeShell, args, 1);
	XtSetArg(args[0], XtNiconPixmap, (XtArgVal) xedit_icon_pixmap);
	XtSetValues(editShell, args, 1);
}	/* initialize_shell_structures */


/*
set_wm_protocols() builds a list of atoms and then sets the
WM_PROTOTCOLS property.  XtConvertAndStore() is an alternative
to XInternAtom(), but we prefer the latter.
*/

static void set_wm_protocols()
{
	Atom atoms[1];
	Display *dpy = XtDisplay(topLevel);

	WM_DELETE_WINDOW = atoms[0] = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
	if (!XSetWMProtocols(dpy, XtWindow(topLevel), atoms, XtNumber(atoms)) ||
	!XSetWMProtocols(dpy, XtWindow(compileShell), atoms, XtNumber(atoms)) ||
	!XSetWMProtocols(dpy, XtWindow(editShell), atoms, XtNumber(atoms)) ||
	!XSetWMProtocols(dpy, XtWindow(errorShell), atoms, XtNumber(atoms)) ||
	!XSetWMProtocols(dpy, XtWindow(makeShell), atoms, XtNumber(atoms)) ||
	!XSetWMProtocols(dpy, XtWindow(yncShell), atoms, XtNumber(atoms)) ||
	!XSetWMProtocols(dpy, XtWindow(contShell), atoms, XtNumber(atoms)))
		fprintf(stderr,
			"\nxcctool: can't intern/register the WM_PROTOCOLS atom.\n");
}	/* set_wm_protocols */


/*
process_wm_event() checks for an event of interest from the
window manager.  Currently, it only handles WM_DELETE_WINDOW
messages.
*/

static int process_wm_event(event)
XEvent *event;
{
	if (!event->xclient.send_event)
		return FALSE;
	if (event->xclient.window == XtWindow(topLevel) ||
			event->xclient.window == XtWindow(compileShell) ||
			event->xclient.window == XtWindow(editShell) ||
			event->xclient.window == XtWindow(errorShell) ||
			event->xclient.window == XtWindow(makeShell) ||
			event->xclient.window == XtWindow(yncShell) ||
			event->xclient.window == XtWindow(contShell)) {
		static XWindowAttributes xwa;

		XGetWindowAttributes(XtDisplay(topLevel), event->xclient.window, &xwa);
		if (xwa.map_state != IsUnmapped) {
			if (event->xclient.data.l[0] == WM_DELETE_WINDOW) {
				if (event->xclient.window == XtWindow(topLevel))
					Quit(topLevel, NULL, NULL);
				else if (event->xclient.window == XtWindow(editShell))
					EditDone(NULL, NULL, NULL);
				else if (event->xclient.window == XtWindow(yncShell))
					alert_delete(yncShell);
				else if (event->xclient.window == XtWindow(contShell))
					alert_delete(contShell);
				else if (event->xclient.window == XtWindow(compileShell))
					XtUnmapWidget(compileShell);
				else if (event->xclient.window == XtWindow(errorShell))
					XtUnmapWidget(errorShell);
				else if (event->xclient.window == XtWindow(makeShell))
					XtUnmapWidget(makeShell);
			}
		}
		return TRUE;
	}
	else
		return FALSE;
}	/* process_wm_event */


/*
Make these public so that other modules can map the secondary windows:
*/

/*
compile() is the starting point for compiler operations.
*/

void compile()
{
	XtMapWidget(compileShell);
}	/* compile */


/*
error() is the starting point for error pop-up operations.
*/

void error()
{
	XtMapWidget(errorShell);
}	/* error */


/*
make() is the starting point for make operations.
*/

void make()
{
	XtMapWidget(makeShell);
}	/* make */


/*
Action functions:
*/

/*
WMProtocolsAction() currently handles ClientMessage events.
*/

/*ARGSUSED*/
static void WMProtocolsAction(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
	process_wm_event(event);
}	/* WMProtocolsAction */


/*
Callback functions:
*/

/*
Quit() terminates the application.
*/
/*ARGSUSED*/
static void Quit(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if (modified_edit_buffer())
		edit();
	if (!check_save_edit_buffer())
		return;
	XFreePixmap(XtDisplay(w), xcctool_icon_pixmap);
	XFreePixmap(XtDisplay(w), xcompile_icon_pixmap);
	XFreePixmap(XtDisplay(w), xerror_icon_pixmap);
	XFreePixmap(XtDisplay(w), xmake_icon_pixmap);
	XFreePixmap(XtDisplay(w), xedit_icon_pixmap);
	cleanup_edit_window();
	exit(0);
}	/* Quit */


/*
Compile() invokes the compiler pop-up window.
*/
/*ARGSUSED*/
static void Compile(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	compile();
}	/* Compile */


/*
CompileDone() removes the "compile" window.
*/
/*ARGSUSED*/
static void CompileDone(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XtUnmapWidget(compileShell);
}	/* CompileDone */


/*
Make() invokes the make pop-up window.
*/
/*ARGSUSED*/
static void Make(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	make();
}	/* Make */


/*
MakeDone() removes the "make" window.
*/
/*ARGSUSED*/
static void MakeDone(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XtUnmapWidget(makeShell);
}	/* MakeDone */


/*
CompileError() invokes the error pop-up window.
*/
/*ARGSUSED*/
static void CompileError(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	error();
}	/* CompileError */


/*
ErrorDone() removes the "error" window.
*/
/*ARGSUSED*/
static void ErrorDone(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XtUnmapWidget(errorShell);
}	/* ErrorDone */


/*
CompileCompile() starts the compilation.
*/
/*ARGSUSED*/
static void CompileCompile(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	compile_file();
}	/* CompileCompile */


/*
CompileEdit() edits the file specified in the
directory and filename boxes.
*/
/*ARGSUSED*/
static void CompileEdit(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if (edit_text())
		edit();
}	/* CompileEdit */


/*
PreviousError() finds the previous error.
*/
/*ARGSUSED*/
static void PreviousError(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	process_previous_error();
}	/* PreviousError */


/*
NextError() finds the next error.
*/
/*ARGSUSED*/
static void NextError(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	process_next_error();
}	/* NextError */


/*
MakeMake() starts the make operation.
*/
/*ARGSUSED*/
static void MakeMake(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	make_file();
}	/* MakeMake */


