.TH XCCTOOL L "1 December 1990"
.SH NAME
xcctool \- X Window System window- and mouse-based compiler utility
.SH SYNOPSIS
.B xcctool
.SH DESCRIPTION
.B xcctool
is a compiler tool for C programmers that operates under the
X Window System.
.B xcctool
provides a semi-integrated programming environment that automates
and unifies many of the routine operations that a C programmer normally
conducts via a command window.
.B xcctool
uses panels, text-entry areas, and buttons to automate the use
of cc(1) and make(1) \- without restricting the
programmer to prescribed operations.
.LP
In addition to automating the specification of filenames and other
options (parameters) for compilation and make activities,
.B xcctool
provides error tracking during compilation using an error window and an
edit window.
.SH USAGE
Typically,
.B xcctool
would be invoked from a command window, or from an X Window System
background menu, if provided by your system.
From the command line, to invoke
.B xcctool
as a background process you would enter:

{system:43} xcctool &

where "{system:43}" is the command window prompt.  You can override
the default window parameters for
.B xcctool
using command line arguments or by setting options in a resource file.
.LP
.I Window and Panel Organization

.B xcctool's
top-level window is very small, containing only three buttons.  The
first button
.B [Quit]
terminates
.B xcctool
The
.B [Compile]
and
[Make]
buttons activate top-level windows related to these tasks.
The compilation window also has buttons/commands that invoke
error-tracking in an edit window.
.LP
.I Edit and Compile Facilities

Typically, edit and compilation operations go hand in hand.  With
.B xcctool
you can edit a source file using a toolkit-based edit window, compile
the source file, continue editing the same file or switch to another
file, and so on.  In addition, if there are errors during compilation,
an error window pops up displaying the error message.
.LP
Within the compilation window,
.B [Compile]
invokes the C compiler with the options specified in the
.B cc options
text area.  You can specify the compiler name as well, using the
.B Compiler
text area.
.B xcctool
has been tested with
.B cc,
.B gcc,
and
.B g++.
(
.B gcc
(tm) and
.B g++
(tm) are distributed by the Free Software Foundation.)
.LP
If the compiler options do not include "-c", and there are no errors,
the linker will build an executable file in "a.out".  In this case, the
.B Libraries
text area is relevant \- the libraries specified here will be used to
build the executable; see below.
With projects that span multiple source files, the compilation window
is used to develop individual object files and the make window
is used to build an executable file.  Lastly, you can use the
.B Directory
and
.B Filename
text areas to specify the location/name of the source file
for compilation.
.LP
.B [Compile]
uses output from the compiler to track errors across ".h"
and ".c" files.  In some cases, an error message returned by the
compiler may not include a filename, may not include a line number,
or may not include either of the two.  In situations like this, just
the error message is displayed in the pop-up error window.
.LP
Typically, however, both a filename and a line number are present in
the error message, and
.B [Compile]
opens the appropriate file for editing with the cursor on the
referenced line.  In this case,
.B [Previous Error]
and
.B [Next Error]
move you backward and forward through your errors, from one source
file to another, if necessary.
.LP
If a file switch is required by
.B [Previous Error]
or
.B [Next Error],
you are given an opportunity to save any unsaved changes before the
switch takes place.  If you make changes to a file that modify the
line number of statements that were flagged with errors,
.B [Compile]
cannot automatically make adjustments in the line numbers referenced
by the compiler.  Typically, line number discrepancies aren't a problem,
since the cursor positioning will be "off" by only a small distance.
.LP
There are other occasions when you are prompted regarding unsaved
changes.  For example, if you use
.B [Edit]
to make changes to the source file, and then click left on
.B [Compile]
without explicitly saving the current edit file, you are given an
opportunity to save before the compilation.  Thus, you can save
keystrokes (mousestrokes) by clicking left on
.B [Compile]
without saving beforehand.
.LP
As another example, suppose that you (1) makes changes to the
current edit file without saving them, (2) type/select a new
filename, and (3) click left on
.B[Edit].
Again, you will be allowed to save before the new edit file is
loaded \- the auto-prompting for unsaved text prevents accidental losses
of edited text (and saves keystrokes).
.LP
Note:
.B [Make]
does
.I not
check for unsaved changes to the current edit file before making the
project.
.LP
Note that if you're using
.B [Compile]
to compile and link a single source file, the programmer's libraries
given in the pop-up window for cc(1) options are automatically passed
to the linker.  You can use the mouse to select and delete these library
names, if the default libraries aren't needed, or to make changes in the
specified library names.
.B [Make]
also uses these libraries, if the proper options are selected in the
make options pop-up window.
.LP
.LP
.I The Make Facility

From the top-level window,
.B [Make]
invokes the make top-level window.  Within the make window,
.B [Make]
invokes make(1) with the options specified in the
.B make options
text area.  The make file should be specified in the
.B Directory
and
.B Filename
text areas.  If no file is specified,
.B [Make]
builds the make command without a filename; hence, the standard
make(1) defaults for the make filename are employed.
.LP
The make pop-up window provides other options to assist with various
make scenarios.  For example, if your make file uses either or both of
the macros"$(LDFLAGS)" and "$(CFLAGS)", you can request that
information from the compile options window be passed to these macros.
The
.B $(LDFLAGS) options
text area allows you to pass additional linker/loader options to your
make file, beyond those specified in the
.B Libraries
text area of the compilation window.
.LP
Note:
.B [Make]
does
.I not
check for unsaved changes to the current edit file before making the
project.
.LP
.LP
.I Resources

If you are experienced with setting X resources \- see an X Window
System user's guide \- you can control various aspects of
.B xcctool's
appearance.
.B xcctool
has four window-related resources that can be used to alter
characteristics of the edit window that's
invoked with
.B [Edit].
These application resource names, classes, and default values are:
.LP
.in +4
- editRows/EditRows \- 30
.LP
.in +4
- editColumns/EditColumns \- 80
.LP
.in +4
- editSize/EditSize \- 50000 (characters)
.LP
.in +4
- editFont/EditFont \- fixed
.LP
A final application resource, "libraries" (class:  "Libraries"),
controls the list of libraries (at program start-up) that's
specified in the
.B Libraries 
text area of the compilation window.
.LP
If you would like a list of the external names of the widgets that
are used in
.B xcctool's
interface, see the hard-copy documentation that accompanies
.B xcctool.
.SH AUTHOR
.PP
Jerry Smith, Iris Computing Laboratories
.PP
Copyright 1990, Iris Computing Laboratories
